@@ -3,17 +3,124 @@ from typing import Sequence, Union, List, Tuple
 from sqlexec.table_exec import get_condition_arg, get_where_arg_limit
 
 
+class FieldWherePageExec:
+    def __init__(self, field_where_exec, page_num, page_size):
+        self.field_where_exec = field_where_exec
+        self.page_num = page_num
+        self.page_size = page_size
+
+    def find(self):
+        return self.field_where_exec.find_page(self.page_num, self.page_size)
+
+    def query(self):
+        return self.field_where_exec.query_page(self.page_num, self.page_size)
+
+    def select(self):
+        return self.field_where_exec.select_page(self.page_num, self.page_size)
+
+
+class FieldWhereExec:
+    def __init__(self, field_exec, **kwargs):
+        self.field_exec = field_exec
+        self.kwargs = kwargs
+
+    def find(self):
+        """
+        Return list(object) or empty list if no result.
+        persons = Person.fields('id', 'name', 'age').where(name='张三', age=55).find()
+        """
+        return self.field_exec.find(**self.kwargs)
+
+    def find_one(self):
+        """
+        Return unique result(object) or None if no result.
+        person = Person.fields('id', 'name', 'age').where(name='张三', age=55).find_one()
+        """
+        return self.field_exec.find_one(**self.kwargs)
+
+    def query(self):
+        """
+        Return list(dict) or empty list if no result.
+        persons = Person.fields('id', 'name', 'age').where(name='张三', age=55).query()
+        """
+        return self.field_exec.query(**self.kwargs)
+
+    def query_one(self):
+        """
+        Return unique result(dict) or None if no result.
+        persons = Person.fields('id', 'name', 'age').where(name='张三', age=55).query_one()
+        """
+        return self.field_exec.query_one(**self.kwargs)
+
+    def select(self):
+        """
+        Return list(dict) or empty list if no result.
+        rows = Person.fields('id', 'name', 'age').where(name='张三', age=55).select()
+        """
+        return self.field_exec.select(**self.kwargs)
+
+    def select_one(self):
+        """
+        Return unique result(tuple) or None if no result.
+        row = Person.fields('id', 'name', 'age').where(name='张三', age=55).select_one()
+        """
+        return self.field_exec.select_one(**self.kwargs)
+
+    def find_page(self, page_num=1, page_size=10):
+        """
+        Return list(object) or empty list if no result.
+        persons = Person.fields('id', 'name', 'age').where(name='张三', age=55).find_page(1, 10)
+        :param page_num: page number
+        :param page_size: page size
+        """
+        return self.field_exec.find_page(page_num, page_size, **self.kwargs)
+
+    def query_page(self, page_num=1, page_size=10):
+        """
+        Return list(dict) or empty list if no result.
+        persons = Person.fields('id', 'name', 'age').where(name='张三', age=55).query_page(1, 10)
+        :param page_num: page number
+        :param page_size: page size
+        """
+        return self.field_exec.query_page(page_num, page_size, **self.kwargs)
+
+    def select_page(self, page_num=1, page_size=10):
+        """
+        Return list(dict) or empty list if no result.
+        rows = Person.fields('id', 'name', 'age').where(name='张三', age=55).select_page(1, 10)
+        :param page_num: page number
+        :param page_size: page size
+        """
+        return self.field_exec.select_page(page_num, page_size, **self.kwargs)
+
+    def page(self, page_num=1, page_size=10) -> FieldWherePageExec:
+        return FieldWherePageExec(self, page_num, page_size)
+
+
+class FieldPageExec:
+    def __init__(self, field_exec, page_num=1, page_size=10):
+        self.field_exec = field_exec
+        self.page_num = page_num
+        self.page_size = page_size
+
+    def find(self, **kwargs):
+        return self.field_exec.find_page(self.page_num, self.page_size, **kwargs)
+
+    def query(self, **kwargs):
+        return self.field_exec.query_page(self.page_num, self.page_size, **kwargs)
+
+    def select(self, **kwargs):
+        return self.field_exec.select_page(self.page_num, self.page_size, **kwargs)
+
+    def where(self, **kwargs) -> FieldWherePageExec:
+        return FieldWherePageExec(FieldWhereExec(self.field_exec, **kwargs), self.page_num, self.page_size)
+
+
 class FieldExec:
     def __init__(self, cls, *fields):
         self.cls = cls
         self.fields = fields
 
-    def where(self, **kwargs):
-        return WhereExec(self.cls, *self.fields, **kwargs)
-
-    def page(self, page_num=1, page_size=10):
-        return OrmPage(self.where(), page_num, page_size)
-
     def find(self, **kwargs):
         """
         Return list(object) or empty list if no result.
@@ -28,13 +135,6 @@ class FieldExec:
         """
         return self.cls.find_one(*self.fields, **kwargs)
 
-    # def find_by(self, where: str, *args, **kwargs):
-    #     """
-    #     Return list(dict) or empty list if no result.
-    #     rows = Person.fields('id', 'name', 'age').find_by('where name=?', '李四')
-    #     """
-    #     return [self.cls.to_obj(**d) for d in self.query_by(where, *args, **kwargs)]
-
     def find_by_id(self, _id: Union[int, str]):
         """
         Return one class object or None if no result.
@@ -65,14 +165,6 @@ class FieldExec:
         """
         return self.cls.query_one(*self.fields, **kwargs)
 
-    # def query_by(self, where: str, *args, **kwargs):
-    #     """
-    #     Return list(dict) or empty list if no result.
-    #     rows = Person.fields('id', 'name', 'age').query_by('where name=?', '李四')
-    #     """
-    #     sql, args = self._get_by_sql_args(where, *args, **kwargs)
-    #     return do_query(sql, *args)
-
     def query_by_id(self, _id: Union[int, str]):
         """
         Return one row(dict) or None if no result.
@@ -103,15 +195,6 @@ class FieldExec:
         """
         return self.cls.select_one(*self.fields, **kwargs)
 
-    # def select_by(self, where: str, *args, **kwargs):
-    #     """
-    #     Return list(dict) or empty list if no result.
-    #     rows = Person.select_by_where('where name=?', '李四')
-    #     """
-    #     assert where and where.strip().lower().startswith('where'), "Parameter 'where' must startswith 'WHERE'"
-    #     sql, args = self._get_by_sql_args(where, *args, **kwargs)
-    #     return do_select(sql, *args)
-
     def select_by_id(self, _id: Union[int, str]):
         """
         Return one row(dict) or None if no result.
@@ -138,13 +221,6 @@ class FieldExec:
         """
         return self.cls.find_page(page_num, page_size, *self.fields, **kwargs)
 
-    # def find_page_by(self, page_num: int, page_size: int, where: str, *args, **kwargs):
-    #     """
-    #     Return list(dict) or empty list if no result. Automatically add 'limit ?,?' after where if not.
-    #     rows = Person.find_by_page(1, 10, 'where name=?', '李四')
-    #     """
-    #     return [self.cls.to_obj(**d) for d in self.query_page_by(page_num, page_size, where, *args, **kwargs)]
-
     def query_page(self, page_num=1, page_size=10, **kwargs):
         """
         Return list(dict) or empty list if no result.
@@ -154,15 +230,6 @@ class FieldExec:
         """
         return self.cls.query_page(page_num, page_size, *self.fields, **kwargs)
 
-    # def query_page_by(self, page_num: int, page_size: int, where: str, *args, **kwargs):
-    #     """
-    #     Return list(dict) or empty list if no result. Automatically add 'limit ?,?' after where if not.
-    #     rows = Person.fields('id', 'name', 'age').query_by_page(1, 10, 'where name=?', '李四')
-    #     """
-    #     assert where and where.strip().lower().startswith('where'), "Parameter 'where' must startswith 'WHERE'"
-    #     sql, args = self._get_by_sql_args(where, *args, **kwargs)
-    #     return do_query_page(sql, page_num, page_size, *args)
-
     def select_page(self, page_num=1, page_size=10, **kwargs):
         """
         Return list(dict) or empty list if no result.
@@ -172,131 +239,148 @@ class FieldExec:
         """
         return self.cls.select_page(page_num, page_size, *self.fields, **kwargs)
 
-    # def select_page_by(self, page_num: int, page_size: int, where: str, *args, **kwargs):
-    #     """
-    #     Return list(dict) or empty list if no result. Automatically add 'limit ?,?' after where if not.
-    #     rows = Person.fields('id', 'name', 'age').select_by_page(1, 10, 'where name=?', '李四')
-    #     """
-    #     assert where and where.strip().lower().startswith('where'), "Parameter 'where' must startswith 'WHERE'"
-    #     sql, args = self._get_by_sql_args(where, *args, **kwargs)
-    #     return do_select_page(sql, page_num, page_size, *args)
+    def where(self, **kwargs) -> FieldWhereExec:
+        return FieldWhereExec(self, **kwargs)
+
+    def page(self, page_num=1, page_size=10) -> FieldPageExec:
+        return FieldPageExec(self, page_num, page_size)
+
+
+class WherePageExec:
+    def __init__(self, where_exec, page_num=1, page_size=10):
+        self.where_exec = where_exec
+        self.page_num = page_num
+        self.page_size = page_size
+
+    def select(self, *fields):
+        """
+        Return list(dict) or empty list if no result.
+        rows = Person.where(name='张三', age=55).page(1, 10).select()
+        """
+        return self.where_exec.select_page(self.page_num, self.page_size, *fields)
+
+    def query(self, *fields):
+        """
+        Return list(dict) or empty list if no result.
+        rows = Person.where(name='张三', age=55).page(1, 10).query()
+        """
+        return self.where_exec.query_page(self.page_num, self.page_size, *fields)
+
+    def find(self, *fields):
+        """
+        Return list(dict) or empty list if no result.
+        rows = Person.where(name='张三', age=55).page(1, 10).find()
+        """
+        return self.where_exec.find_page(self.page_num, self.page_size, *fields)
+
+    def fields(self, *fields) -> FieldWherePageExec:
+        return FieldWherePageExec(FieldWhereExec(FieldExec(self.where_exec.cls, *fields), **self.where_exec.kwargs), self.page_num, self.page_size)
 
 
 class WhereExec:
-    def __init__(self, cls, *fields, **kwargs):
+    def __init__(self, cls, **kwargs):
         self.cls = cls
-        self._fields = fields
         self.kwargs = kwargs
 
-    def fields(self, *fields):
-        self._fields = fields
-        return self
-
-    def where(self, **kwargs):
-        self.kwargs = kwargs
-        return self
-
-    def page(self, page_num=1, page_size=10):
-        return OrmPage(self, page_num, page_size)
-
     def delete(self):
         """
         Physical delete
-        rowcount = Person.delete_by('where name=? and age=?', '张三', 55)
+        rowcount = Person.where(name='张三', age=55).delete()
         return: Effect rowcount
         """
         return self.cls.delete(**self.kwargs)
 
     def count(self):
         """
-        Automatically add 'limit ?' where if not.
-        count = Person.count_by('where name=?', '李四')
+        count = Person.where(name='张三', age=55).count()
         """
         return self.cls.count(**self.kwargs)
 
     def exists(self):
+        """
+        flag = Person.where(name='张三', age=55).exists()
+        """
         return self.cls.exists(**self.kwargs)
 
-    def select(self):
+    def select(self, *fields):
         """
         Return list(dict) or empty list if no result.
-        rows = Person.where('where name=?').select('李四')
+        rows = Person.where(name='张三', age=55).select()
         """
-        return self.cls.select(*self._fields, **self.kwargs)
+        return self.cls.select(*fields, **self.kwargs)
 
-    def query(self):
+    def query(self, *fields):
         """
         Return list(dict) or empty list if no result.
-        rows = Person.where('where name=?').query('李四')
+        rows = Person.where(name='张三', age=55).query()
         """
-        return self.cls.query(*self._fields, **self.kwargs)
+        return self.cls.query(*fields, **self.kwargs)
 
-    def find(self):
+    def find(self, *fields):
         """
         Return list(dict) or empty list if no result.
-        rows = Person.where('where name=?').find('李四')
+        rows = Person.where(name='张三', age=55).find()
         """
-        return [self.cls.to_obj(**d) for d in self.query(*self._fields, **self.kwargs)]
+        return self.cls.find(*fields, **self.kwargs)
 
-    def select_page(self, page_num=1, page_size=10):
+    def select_page(self, page_num=1, page_size=10, *fields):
         """
         Return list(dict) or empty list if no result.
-        rows = Person.where('where name=?').select('李四')
+        rows = Person.where(name='张三', age=55).select_page()
         """
-        return self.cls.select_page(*self._fields, **self.kwargs)
+        return self.cls.select_page(page_num, page_size, *fields, **self.kwargs)
 
-    def query_page(self, page_num=1, page_size=10):
+    def query_page(self, page_num=1, page_size=10, *fields):
         """
         Return list(dict) or empty list if no result.
-        rows = Person.where('where name=?').query('李四')
+        rows = Person.where(name='张三', age=55).query_page()
         """
-        return self.cls.query_page(*self._fields, **self.kwargs)
+        return self.cls.query_page(page_num, page_size, *fields, **self.kwargs)
 
-    def find_page(self, page_num=1, page_size=10):
+    def find_page(self, page_num=1, page_size=10, *fields):
         """
         Return list(dict) or empty list if no result.
-        rows = Person.where('where name=?').find('李四')
+        rows = Person.where(name='张三', age=55).find_page()
         """
-        return [self.cls.to_obj(**d) for d in self.query_page(page_num, page_size)]
+        return self.cls.find_page(page_num, page_size, *fields, **self.kwargs)
+
+    def fields(self, *fields) -> FieldWhereExec:
+        return FieldWhereExec(FieldExec(self.cls, *fields), **self.kwargs)
+
+    def page(self, page_num=1, page_size=10) -> WherePageExec:
+        return WherePageExec(self, page_num, page_size)
 
 
-class OrmPage:
+class PageExec:
     
-    def __init__(self, exec: WhereExec, page_num=1, page_size=10):
-        self.exec = exec
+    def __init__(self, cls, page_num=1, page_size=10):
+        self.cls = cls
         self.page_num = page_num
         self.page_size = page_size
 
-    def query(self, *fields, **kwargs):
-        """
-        Execute select SQL and return list or empty list if no result. Automatically add 'limit ?,?' after sql statement if not.
-        sql: SELECT * FROM user WHERE name=? and age=?  -->  args: ('张三', 20)
-             SELECT * FROM user WHERE name=:name and age=:age  -->  kwargs: ('张三', 20) --> kwargs: {'name': '张三', 'age': 20}
-        """
-        return self.exec.cls.query_page(self.page_num, self.page_size, *fields, **kwargs)
-
     def select(self, *fields, **kwargs):
         """
-        Execute select SQL and return list(tuple) or empty list if no result. Automatically add 'limit ?,?' after sql statement if not.
-        sql: SELECT * FROM user WHERE name=? and age=?  -->  args: ('张三', 20)
-             SELECT * FROM user WHERE name=:name and age=:age   -->  kwargs: ('张三', 20) --> kwargs: {'name': '张三', 'age': 20}
+        Person.page(1, 10).select('id', 'name', 'age', name='张三', age=55)
         """
-        return self.exec.cls.select_page(self.page_num, self.page_size, *fields, **kwargs)
+        return self.cls.select_page(self.page_num, self.page_size, *fields, **kwargs)
 
-    def where(self, **kwargs):
-        self.exec.kwargs = kwargs
-        return self
-
-    def fields(self, *fields) -> WhereExec:
-        return self.exec.fields(*fields)
-
-    def find(self, *args, **kwargs):
+    def query(self, *fields, **kwargs):
         """
-        Execute select SQL and return list or empty list if no result. Automatically add 'limit ?,?' after sql statement if not.
-        sql: SELECT * FROM user WHERE name=? and age=?  -->  args: ('张三', 20)
-             SELECT * FROM user WHERE name=:name and age=:age  -->  kwargs: ('张三', 20) --> kwargs: {'name': '张三', 'age': 20}
+        Person.page(1, 10).query('id', 'name', 'age', name='张三', age=55)
         """
-        return self.exec.cls.find_page(self.page_num, self.page_size, *fields, **kwargs)
+        return self.cls.query_page(self.page_num, self.page_size, *fields, **kwargs)
+
+    def find(self, *fields, **kwargs):
+        """
+        Person.page(1, 10).query('id', 'name', 'age', name='张三', age=55)
+        """
+        return self.cls.find_page(self.page_num, self.page_size, *fields, **kwargs)
+
+    def where(self, **kwargs) -> WherePageExec:
+        return WherePageExec(WhereExec(self.cls, **kwargs), self.page_num, self.page_size)
+
+    def fields(self, *fields) -> FieldPageExec:
+        return FieldPageExec(FieldExec(self.cls, *fields), self.page_num, self.page_size)
 
 
 def split_ids(ids: Sequence[int], batch_size):
